
# q-expansion of newform 6566.2.a.b, downloaded from the LMFDB on 14 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 6566
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [-2], [4], [0], [4], [4], [4], [0], [0], [10], [-8], [2], [10], [-4], [2], [-6], [4], [8], [-1], [0], [4], [-4], [8], [0], [-14], [-12], [-14], [8], [-2], [10], [16], [0], [-2], [-10], [22], [16], [-22], [8], [6], [-18], [12], [-10], [-12], [2], [-26], [-22], [-8], [-14], [-20], [-8], [-26], [20], [-8], [-30], [24], [-16], [-6], [-8], [-26], [22], [-24], [18], [4], [-28], [-18], [-10], [4], [-10], [-12], [10], [14], [24], [0], [34], [-28], [16], [2], [2], [-18], [14], [16], [38], [32], [-6], [34], [-20], [30], [-2], [18], [36], [-20], [30], [32], [8], [12], [20], [-6], [14], [0], [-2], [28], [-42], [-30], [-30], [28], [-2], [18], [-26], [-16], [16], [-22], [26], [22], [-4], [-24], [22], [-48], [-32], [14], [36], [4], [-34], [24], [36], [-36], [-30], [-6], [-6], [-4], [48], [-4], [-24], [24], [-22], [-12], [26], [54], [18], [6], [2], [14], [46], [40], [-44], [-34], [10], [2], [-18], [-20], [-48], [-22], [0], [-36], [-54], [-24], [-48], [24], [10], [14], [0], [-4], [42], [-8], [12], [-30], [60], [60], [-18], [-34], [-24], [-36], [-14], [32], [62], [-44], [58], [44], [-6], [-28], [-38], [16], [48], [-38], [-12], [0], [-22], [-18], [2], [-26], [-12], [56], [-52], [-20], [60], [-4], [-16], [-18], [-62], [-14], [-22], [54], [-26], [-12], [58], [14], [22], [30], [-24], [-54], [2], [58], [-32], [20], [-58], [24], [-2], [24], [-60], [12], [26], [58], [74], [-30], [16], [2], [-22], [12], [60], [58], [44], [-50], [42], [24], [-6], [-8], [-48], [24], [-26], [36], [-74], [8], [-50], [-4], [-18], [-54], [-36], [-22], [-6], [36], [12], [38], [28], [12], [-4], [12], [4], [42], [4], [14], [68], [40], [-52], [68], [-52], [-60], [40], [14], [50], [-44], [-26], [-36], [20], [22], [64], [12], [-20], [4], [-10], [-6], [-62], [18], [-32], [-56], [0], [-46], [-48], [38], [-18], [60], [-14], [-2], [-14], [-62], [48], [-6], [-58], [-48], [-16], [-24], [-26], [6], [2], [16], [68], [16], [-34], [-12], [82], [-64], [38], [-28], [26], [-54], [36], [-40], [18], [32], [40], [-44], [6], [-16], [26], [-90], [-92], [-42], [-28], [52], [28], [-64], [-2], [-42], [18], [-76], [30], [88], [-48], [-94], [24], [-30], [90], [70], [6], [44], [-32], [50], [-36], [4], [-52], [2], [64], [-64], [-54], [54], [44], [-90], [30], [42], [-30], [26], [-76], [62], [-48], [-56], [-20], [80], [6], [-98], [-72], [-62], [84], [80], [24], [66], [60], [-54], [8], [52], [-68], [78], [-8], [-28], [-30], [-24], [2], [-38], [32], [-52], [-46], [84], [-96], [90], [-88], [64], [10], [-40], [74], [2], [-64], [28], [54], [40], [22], [-28], [-30], [-64], [24], [10], [14], [-56], [6], [24], [42], [64], [48], [-26], [-90], [40], [22], [-72], [-46], [6], [-22], [68], [-14], [-58], [-34], [76], [-10], [16], [0]]
